@make(ais)
@aiInfo(topic=<Raising an exception in an abnormally completed task>,
number=[AI-00446/05],
classcode=[BI],
statuscode=[WJ],
mainref=[09.10@\(06)],
date=[87-09-25])
@aiHeading()
@begin(history)
@begin(toplines)
@mainref()
@standard[11.04@\(01)]
!class binding interpretation 86-07-10
!status approved by WG9/AJPO 87-07-29
@end(toplines)
!status approved by Director, AJPO 87-07-29
!status approved by Ada Board 87-07-29
!status approved by WG9 87-05-29
!status panel/committee-approved 87-01-19 (reviewed)
!status panel/committee-approved (6-0-4) 86-11-13 (pending editorial review)
!status work-item 86-08-12
!status received 86-07-10
@references[!references @CommentNum[83-00772]83-00772]
@topic{!topic @value(aitopic)}
@end(history)

@subheading[!summary 86-12-18]

An exception can be propagated to an abnormally completed task that is
engaged in a rendezvous or that is waiting for a task to be activated.
If this occurs, the exception has no effect.

@subheading[!question 87-09-25]

11.4(1) says:
@begin(quotation)
When  an  exception  is  raised,  normal  program  execution is
abandoned and control is transferred to an exception handler.
@end(quotation)
This wording contains an implicit assumption that an exception can only be
raised in a task that has not completed its execution, but the Standard
allows an exception to be raised in a task that has completed its execution
abnormally.  To see how this can occur, consider the following examples.
First consider a task that calls an entry and becomes abnormal while in a
rendezvous.  9.10(6) says:
@begin(quotation)
If a  task that  calls an  entry becomes  abnormal while  in  a
rendezvous, its  termination does  not  take place  before  the
completion of the rendezvous.
@end(quotation)
Although the calling task cannot terminate before the rendezvous is finished,
9.10(6) allows completion of the calling task while the rendezvous is being
performed, i.e., although the storage associated with the calling task cannot
be reclaimed while the rendezvous is in progress, the task can be marked as
requiring no further execution.  If an exception is raised in the rendezvous,
it is sent to the calling task after it has become abnormal and after it has
become complete:
@begin(example)
@key[task] CALLER;
@hinge()

@key[task] @key[body] CALLER @key[is]
     @key[task] SERVER @key[is]
          @key[entry] REQ;
     @key[end] SERVER;
@hinge()

     @key[task] @key[body] SERVER @key[is]
     @key[begin]
          @key[accept] REQ @key[do]
               @key[abort] CALLER;
               @key[raise] PROGRAM_@|ERROR;     -- send exception to caller
          @key[end] REQ;
     @key[end] SERVER;
@hinge()
@key[begin]
     SERVER.REQ;
          -- during the rendezvous, CALLER becomes abnormal and
          -- the rendezvous is completed with an exception
@key[exception]
     @key[when] @key[others] =>
          -- some action
@key[end] CALLER;
@end(example)
If CALLER completes abnormally during the rendezvous, what is the effect of
propagating the exception?

A similar example can be constructed in which a task becomes abnormal
while executing an allocator and activating a task:
@begin(example)
@key[task] @key[body] ACTIVATOR @key[is]
     ACC_@|TSK : ACC_@|SERVER;
@key[begin]
     ACC_@|TSK := @key[new] SERVER;   -- aborts ACTIVATOR
@end(example)
Suppose that during the activation of SERVER, ACTIVATOR becomes abnormal
(because SERVER aborts ACTIVATOR) and in addition, TASKING_@|ERROR is
propagated to ACTIVATOR because SERVER is not successfully activated.
The body of SERVER could be:
@begin(example)
@key[task] @key[body] SERVER @key[is]
     @key[package] P @key[is] @key[end] P;
     @key[package] @key[body] P @key[is]
     @key[begin]
          @key[abort] ACTIVATOR;
          @key[raise] PROGRAM_@|ERROR;
     @key[end] P;
@end(example)
When ACTIVATOR becomes abnormal, it can complete immediately since SERVER is
not dependent on ACTIVATOR.  SERVER's activation continues until
PROGRAM_@|ERROR is raised, and then the unsuccessful activation of SERVER means
TASKING_@|ERROR must be raised in ACTIVATOR.  Since ACTIVATOR is complete, what
is the expected effect?

@subheading[!recommendation 86-08-12]

An attempt to raise an exception in a completed task has no effect.

@subheading[!discussion 86-08-12]

The examples show that it is possible for an exception to be raised in a
task that is complete.  Since the task is complete, the intent is that
the exception be ignored and have no effect.

